#!/usr/bin/env python3
"""
Logging configuration module with rotation and cleanup support.
"""

import logging
import logging.handlers
from pathlib import Path
from datetime import datetime, timedelta
from typing import Optional
from src.config.config import Config


class LoggingManager:
    """Manages logging configuration with rotation and cleanup."""
    
    def __init__(self):
        """Initialize the logging manager."""
        self.logger = None
        self.file_handler = None
        self.console_handler = None
        
    def setup_logging(self) -> logging.Logger:
        """
        Set up comprehensive logging with rotation and cleanup.
        
        Returns:
            Configured logger instance
        """
        # Create logs directory if it doesn't exist
        if Config.LOG_TO_FILE:
            log_dir = Path(Config.LOG_FILE_PATH).parent
            log_dir.mkdir(parents=True, exist_ok=True)
        
        # Create root logger
        logger = logging.getLogger()
        logger.setLevel(getattr(logging, Config.LOG_LEVEL.upper()))
        
        # Clear any existing handlers
        logger.handlers.clear()
        
        # Create formatter
        formatter = logging.Formatter(
            fmt='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
            datefmt='%Y-%m-%d %H:%M:%S'
        )
        
        # Set up console handler
        self.console_handler = logging.StreamHandler()
        self.console_handler.setLevel(getattr(logging, Config.LOG_LEVEL.upper()))
        self.console_handler.setFormatter(formatter)
        logger.addHandler(self.console_handler)
        
        # Set up file handler with rotation (if enabled)
        if Config.LOG_TO_FILE:
            if Config.LOG_ROTATION_TYPE == 'size':
                self.file_handler = logging.handlers.RotatingFileHandler(
                    filename=Config.LOG_FILE_PATH,
                    maxBytes=Config.LOG_MAX_SIZE_MB * 1024 * 1024,  # Convert MB to bytes
                    backupCount=Config.LOG_BACKUP_COUNT,
                    encoding='utf-8'
                )
            else:  # time-based rotation
                self.file_handler = logging.handlers.TimedRotatingFileHandler(
                    filename=Config.LOG_FILE_PATH,
                    when=Config.LOG_ROTATION_INTERVAL,
                    backupCount=Config.LOG_BACKUP_COUNT,
                    encoding='utf-8'
                )
            
            self.file_handler.setLevel(getattr(logging, Config.LOG_LEVEL.upper()))
            self.file_handler.setFormatter(formatter)
            logger.addHandler(self.file_handler)
        
        # Store reference
        self.logger = logger
        
        # Log the configuration
        logger.info("🔧 Logging system initialized")
        logger.info(f"📊 Log Level: {Config.LOG_LEVEL}")
        logger.info(f"📁 Log to File: {Config.LOG_TO_FILE}")
        
        if Config.LOG_TO_FILE:
            logger.info(f"📂 Log File: {Config.LOG_FILE_PATH}")
            logger.info(f"🔄 Rotation Type: {Config.LOG_ROTATION_TYPE}")
            logger.info(f"📏 Max Size: {Config.LOG_MAX_SIZE_MB}MB (size rotation)")
            logger.info(f"🗃️ Backup Count: {Config.LOG_BACKUP_COUNT}")
            logger.info(f"⏰ Rotation Interval: {Config.LOG_ROTATION_INTERVAL} (time rotation)")
        
        return logger
    
    def cleanup_old_logs(self, days_to_keep: int = 30) -> None:
        """
        Clean up old log files beyond the backup count.
        
        Args:
            days_to_keep: Number of days worth of logs to keep
        """
        if not Config.LOG_TO_FILE:
            return
        
        try:
            log_dir = Path(Config.LOG_FILE_PATH).parent
            log_base_name = Path(Config.LOG_FILE_PATH).stem
            
            # Find all log files
            log_files = []
            for file_path in log_dir.glob(f"{log_base_name}*"):
                if file_path.is_file() and file_path.suffix in ['.log', '.log.1', '.log.2']:
                    log_files.append(file_path)
            
            # Sort by modification time (oldest first)
            log_files.sort(key=lambda x: x.stat().st_mtime)
            
            # Calculate cutoff date
            cutoff_date = datetime.now() - timedelta(days=days_to_keep)
            
            # Remove old files
            removed_count = 0
            for log_file in log_files:
                file_date = datetime.fromtimestamp(log_file.stat().st_mtime)
                
                if file_date < cutoff_date:
                    try:
                        log_file.unlink()
                        removed_count += 1
                        if self.logger:
                            self.logger.info(f"🗑️ Removed old log file: {log_file.name}")
                    except OSError as e:
                        if self.logger:
                            self.logger.warning(f"⚠️ Could not remove {log_file.name}: {e}")
            
            if removed_count > 0 and self.logger:
                self.logger.info(f"🧹 Cleanup complete: removed {removed_count} old log files")
            
        except Exception as e:
            if self.logger:
                self.logger.error(f"❌ Error during log cleanup: {e}")
    
    def get_log_stats(self) -> dict:
        """
        Get statistics about current log files.
        
        Returns:
            Dictionary with log file statistics
        """
        stats = {
            'log_to_file': Config.LOG_TO_FILE,
            'log_file_path': Config.LOG_FILE_PATH,
            'current_size_mb': 0,
            'backup_files': 0,
            'total_size_mb': 0
        }
        
        if not Config.LOG_TO_FILE:
            return stats
        
        try:
            log_dir = Path(Config.LOG_FILE_PATH).parent
            log_base_name = Path(Config.LOG_FILE_PATH).stem
            
            # Main log file
            main_log = Path(Config.LOG_FILE_PATH)
            if main_log.exists():
                stats['current_size_mb'] = main_log.stat().st_size / (1024 * 1024)
                stats['total_size_mb'] = stats['current_size_mb']
            
            # Backup files
            backup_files = list(log_dir.glob(f"{log_base_name}*.log.*"))
            stats['backup_files'] = len(backup_files)
            
            # Total size
            for backup_file in backup_files:
                if backup_file.exists():
                    stats['total_size_mb'] += backup_file.stat().st_size / (1024 * 1024)
            
        except Exception as e:
            if self.logger:
                self.logger.error(f"❌ Error getting log stats: {e}")
        
        return stats
    
    def format_log_stats(self, stats: Optional[dict] = None) -> str:
        """
        Format log statistics for display.
        
        Args:
            stats: Log statistics dictionary (will fetch if None)
            
        Returns:
            Formatted string with log statistics
        """
        if stats is None:
            stats = self.get_log_stats()
        
        if not stats['log_to_file']:
            return "📊 <b>Logging:</b> Console only"
        
        return f"""📊 <b>Log File Statistics:</b>
• File: {Path(stats['log_file_path']).name}
• Current Size: {stats['current_size_mb']:.2f} MB
• Backup Files: {stats['backup_files']}
• Total Size: {stats['total_size_mb']:.2f} MB
• Max Size: {Config.LOG_MAX_SIZE_MB} MB
• Rotation: {Config.LOG_ROTATION_TYPE}
• Backups Kept: {Config.LOG_BACKUP_COUNT}"""


# Global logging manager instance
logging_manager = LoggingManager()


def setup_logging() -> logging.Logger:
    """
    Set up logging configuration.
    
    Returns:
        Configured logger instance
    """
    return logging_manager.setup_logging()


def cleanup_logs(days_to_keep: int = 30) -> None:
    """
    Clean up old log files.
    
    Args:
        days_to_keep: Number of days worth of logs to keep
    """
    logging_manager.cleanup_old_logs(days_to_keep)


def get_log_stats() -> dict:
    """Get log file statistics."""
    return logging_manager.get_log_stats()


def format_log_stats(stats: Optional[dict] = None) -> str:
    """Format log statistics for display."""
    return logging_manager.format_log_stats(stats) 